package com.grizzly.test.arp.client;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.concurrent.atomic.AtomicBoolean;

public class StreamRequest
{
	public static final String XML_CONTENT_TYPE = "text/xml";

	public static final String SUB_CHUNK_PARAM = "sub_chunk";

	private static final String VALID_METHODS = "POST,PUT,GET,HEAD,DELETE,OPTIONS,TRACE";

	public static int GET(String resourceUrl, MessageNotifier notifier)
	{
		return GET(resourceUrl, notifier, null);
	}

	public static int GET(String resourceUrl, MessageNotifier notifier, String accept)
	{
		return sendHttpRequest("GET", resourceUrl, null, notifier, null, accept);
	}

	public static int POST(String resourceUrl, Object parameters, MessageNotifier notifier)
	{
		return POST(resourceUrl, parameters, notifier, null, null);
	}

	public static int POST(String resourceUrl, Object parameters, MessageNotifier notifier, String contentType, String accept)
	{
		return sendHttpRequest("POST", resourceUrl, parameters, notifier, contentType, accept);
	}

	public static int sendHttpRequest(String method, String resourceUrl, Object parameters, MessageNotifier notifier, String contentType, String accept)
	{
		final Thread curThread = Thread.currentThread();
		final AtomicBoolean finished = new AtomicBoolean(false);

		HttpURLConnection connection = null;
		try
		{
			if (contentType == null)
			{
				contentType = XML_CONTENT_TYPE;
			}
			if (accept == null)
			{
				accept = contentType;
			}

			if (method == null || VALID_METHODS.indexOf(method) == -1)
			{
				throw new IllegalArgumentException("Invalid HTTP method : '" + (method == null ? "<null>" : method) + "'");
			}

			// Request sub-chuncks to get correctly message bounds.
			resourceUrl = addRequestParameter(resourceUrl, SUB_CHUNK_PARAM, "true");

			connection = (HttpURLConnection) new URL(resourceUrl).openConnection();
			connection.setRequestProperty("accept", accept);
			connection.setRequestProperty("pragma", "no-cache");
			connection.setRequestMethod(method);

			if (method != "GET")
			{
				if (parameters == null)
				{
					throw new IllegalArgumentException("'" + method + "' HTTP method needs non null parameters");
				}

				connection.setDoOutput(true);
				connection.setRequestProperty("content-type", contentType);
				writeStringToStream(parameters.toString(), connection.getOutputStream());
			}

			connection.connect();

			InputStream in = connection.getInputStream();

			ChunkedReader reader = new ChunkedReader(in);
			byte[] bytes;
			while ((bytes = reader.readChunk()) != null)
			{
				ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
				Object obj = readStringFromStream(bis);
				notifier.onMessage(obj);
			}

			int response = connection.getResponseCode();
			return response;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			connection.disconnect();
			finished.getAndSet(true);
		}
		return 404;
	}

	public static String addRequestParameter(String url, String name, String value)
	{
		if (url.indexOf(name) == -1)
		{
			if (url.indexOf("?") == -1)
			{
				url += "?" + name + "=" + value;
			}
			else
			{
				url += "&" + name + "=" + value;
			}
		}
		return url;
	}

	private static String readStringFromStream(InputStream in) throws IOException
	{
		StringBuilder sb = new StringBuilder();
		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		while (reader.ready())
		{
			int i = reader.read();
			sb.append((char) i);
		}
		return sb.toString();
	}

	private static void writeStringToStream(String data, OutputStream out) throws IOException
	{
		out.write(data.getBytes());
	}
}
