package com.grizzly.test.arp.server;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.concurrent.locks.ReentrantLock;

import com.grizzly.test.arp.server.util.StreamingHelper;
import com.grizzly.test.arp.server.util.XmlWriter;
import com.sun.grizzly.tcp.Request;
import com.sun.grizzly.tcp.http11.GrizzlyRequest;
import com.sun.grizzly.tcp.http11.GrizzlyResponse;

public class ResultHandler
{
	private GrizzlyRequest request;

	private boolean needSubChunk;
	private boolean headerWritten = false;

	private XmlWriter writer = new XmlWriter();
	private ReentrantLock writeLock = new ReentrantLock();

	private boolean terminated = false;

	public ResultHandler()
	{
		// Empty constructor
	}

	public void attach(GrizzlyRequest req)
	{
		// System.out.println("ArpHandler.attach: " + attachment);

		this.request = req;
		this.needSubChunk = StreamingHelper.isSubChunkNeeded(this.request);
	}

	public void attach(Request req)
	{
		request = new GrizzlyRequest();
		request.setRequest(req);
		GrizzlyResponse response = new GrizzlyResponse(false, false);
		response.setResponse(req.getResponse());

		// Link objects
		request.setResponse(response);
		response.setRequest(request);

		this.needSubChunk = StreamingHelper.isSubChunkNeeded(this.request);
	}

	public void startResponse() throws IOException
	{
		// Write one message (for IE)
		String str = "Request suspended. ";
		str += generateDummyContentForIE();

		sendData(new AsyncMessage("internal", "info", str));
	}

	public void terminateResponse() throws IOException
	{
		try
		{
			writeLock.lock();

			sendData(new AsyncMessage("internal", "info", "lastMessage"));

			// Terminate the response.
			if (needSubChunk)
			{
				StreamingHelper.streamChunk(null, request.getResponse().getOutputStream(), request.getResponse());
			}

			terminated = true;
		}
		finally
		{
			writeLock.unlock();
		}

		request.getResponse().setStatus(200);

		// Really terminate the request
		Runnable completion = (Runnable) request.getRequest().getAttribute(SimpleArpServer.CALLBACK_KEY);
		completion.run();
	}

	public void sendData(Object data) throws IOException
	{
		try
		{
			writeLock.lock();

			if (!terminated)
			{
				// System.out.println("onEvent: message " + message.getMessage().getId());

				try
				{
					writeData(data);
				}
				catch (IOException e)
				{
					throw e;
				}
				catch (Exception e)
				{
					throw new IOException(e);
				}
			}
			else
			{
				System.out.println("onEvent >>>resumed<<<: message " + data.toString());
			}
		}
		finally
		{
			writeLock.unlock();
		}
	}

	private void writeData(Object data) throws Exception
	{
		GrizzlyResponse response = request.getResponse();
		if (!headerWritten)
		{
			response.setContentType("text/xml");

			response.addHeader("Cache-Control", "private");
			response.addHeader("Pragma", "no-cache");
			response.setCharacterEncoding("UTF-8");

			headerWritten = true;
		}

		ByteArrayOutputStream byteArrayOutput = new ByteArrayOutputStream();
		writer.writeTo(data, byteArrayOutput);
		byteArrayOutput.flush();

		byte[] bytes = byteArrayOutput.toByteArray();
		if (needSubChunk)
		{
			StreamingHelper.streamChunk(bytes, response.getOutputStream(), response);
		}
		else
		{
			response.getOutputBuffer().write(bytes, 0, bytes.length);
		}

		// System.out.println("writeMessage: " + message.getId() + ", size=" + bytes.length);
		response.flushBuffer();
	}

	private String generateDummyContentForIE()
	{
		int breakIEcacheSize = 2 * 1024;

		StringBuilder sb = new StringBuilder();
		sb.append('\r');
		sb.append('\n');
		sb.append("" + breakIEcacheSize + " dummy chars to avoid problems with IE. ");
		sb.append('\r');
		sb.append('\n');
		for (int index = 1; index < breakIEcacheSize; index++)
		{
			sb.append('=');
			if ((index % 80) == 0)
			{
				sb.append('\r');
				sb.append('\n');
				index += 2;
			}
		}
		sb.append('\r');
		sb.append('\n');
		return sb.toString();
	}
}
