package com.grizzly.test.arp.server;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.grizzly.test.arp.server.util.RandomUtil;
import com.sun.grizzly.tcp.ActionCode;
import com.sun.grizzly.tcp.Adapter;
import com.sun.grizzly.tcp.Request;
import com.sun.grizzly.tcp.Response;
import com.sun.grizzly.util.buf.ByteChunk;

class ViewAdapter implements Adapter
{
	private static final ExecutorService exec = Executors.newFixedThreadPool(30);

	public void service(final Request req, final Response res) throws Exception
	{
		if (req.requestURI().toString().equals("/refreshView") == false)
		{
			res.setStatus(400);

			byte[] bytes = ByteChunk.convertToBytes("Bad request !\n");
			ByteChunk chunk = new ByteChunk();
			res.setContentLength(bytes.length);
			res.setContentType("text/plain");
			chunk.append(bytes, 0, bytes.length);
			res.doWrite(chunk);
			return;
		}

		exec.execute(new Refresher(req));
	}

	public void afterService(Request req, Response res) throws Exception
	{
		req.action(ActionCode.ACTION_POST_REQUEST, null);
	}

	public void fireAdapterEvent(String type, Object data)
	{
	}

	private class Refresher implements Runnable
	{
		private final Request req;

		private Refresher(Request req)
		{
			this.req = req;
		}

		@Override
		public void run()
		{
			try
			{
				ResultHandler handler = new ResultHandler();
				handler.attach(req);

				handler.startResponse();

				final int requestCount = 30;
				final String viewId = req.getParameters().getParameter("viewId");
				System.out.println("Refresh for view: " + viewId);

				for (int resultNumber = 0; resultNumber < requestCount; resultNumber++)
				{
					QueryResult result = new QueryResult();
					result.setViewId(viewId);
					result.setName("result" + resultNumber);

					int sleepTime = RandomUtil.nextInt(10 * 100);
					// System.out.println(Thread.currentThread() + ": View" + viewId + ": Sleeping " + sleepTime + " ms");
					Thread.sleep(sleepTime);

					handler.sendData(result);
				}

				handler.terminateResponse();
			}
			catch (Exception ex)
			{
				System.out.println(ex);
			}
		}
	}
}