package com.grizzly.test.arp.server.util;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.nio.charset.Charset;
import java.util.List;
import java.util.Map;

/**
 *
 * @author eVrignaud
 */
public class XmlWriter extends TextWriter
{
	public void writeTo(Object data, OutputStream out) throws IOException
	{
		if (data instanceof String)
		{
			// Data is already in Xml form so write directly on the output
			out.write(data.toString().getBytes());
		}
		else
		{
			// Build an Xml representation of this object
			// To work well this object must be a or some beans.
			PrintStream ps = new PrintStream(out);
			String csn = Charset.defaultCharset().name();
			ps.append("<?xml version=\"1.0\" encoding=\"").append(csn).append("\"?>\n");
			writeObject(data, ps, 0, true);
		}
	}

	private void writeObject(Object data, PrintStream ps, int level, boolean parentIsContainer)
	{
		if (data == null)
		{
			ps.append("null");
		}
		else if (data instanceof Map)
		{
			Map<?, ?> map = (Map<?, ?>) data;
			appendLevel(ps, level).append("<Map>\n");
			for (Object key : map.keySet())
			{
				if (!isNativeType(key))
				{
					throw new IllegalArgumentException("Can't serialise to XML, Map that use complex typed key: '" + key.toString() + "'");
				}
				String keyStr = key.toString();
				appendLevel(ps, level + 1).append("<");
				xmlEscape(ps, keyStr).append(">");
				writeObject(map.get(key), ps, level + 1, false);
				ps.append("</");
				xmlEscape(ps, keyStr).append(">\n");
			}
			appendLevel(ps, level).append("</Map>\n");
		}
		else if (data instanceof List)
		{
			List<?> list = (List<?>) data;
			appendLevel(ps, level).append("<List>\n");
			for (Object item : list)
			{
				writeObject(item, ps, level + 1, true);
			}
			appendLevel(ps, level).append("</List>\n");
		}
		else if (data.getClass().isArray())
		{
			appendLevel(ps, level).append("<Array>\n");
			List<String> list = getPrimitiveArrayAsList(data);
			if (list != null)
			{
				for (String value : list)
				{
					appendLevel(ps, level + 1).append("<value>").append(value).append("</value>\n");
				}
			}
			else
			{
				Object[] array = (Object[]) data;
				for (Object item : array)
				{
					writeObject(item, ps, level + 1, true);
				}
			}
			appendLevel(ps, level).append("</Array>\n");
		}
		else
		{
			beanInfos(data, ps, level, parentIsContainer);
		}
	}

	private void beanInfos(Object obj, PrintStream ps, int level, boolean parentIsContainer)
	{
		Map<?, ?> map = null;
		if (isNativeType(obj))
		{
			if (parentIsContainer)
				ps.append("<value>");
			xmlEscape(ps, obj.toString());
			if (parentIsContainer)
				ps.append("</value>");
		}
		else
		{
			try
			{
				map = ClassHelper.describe(obj);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
			if (map == null)
			{
				if (parentIsContainer)
					ps.append("<value>");
				xmlEscape(ps, obj.toString());
				if (parentIsContainer)
					ps.append("</value>");
				return;
			}

			if (map.size() == 0)
			{
				if (parentIsContainer)
					ps.append("<value>");
				xmlEscape(ps, obj.toString());
				if (parentIsContainer)
					ps.append("</value>");
				return;
			}
			else
			{
				String className = obj.getClass().getSimpleName();
				appendLevel(ps, level).append("<").append(className).append(">\n");
				for (Object key : map.keySet())
				{
					if (!isNativeType(key))
					{
						throw new IllegalArgumentException("Can't serialise to XML, Map that use complex typed key: '" + key.toString() + "'");
					}
					String keyStr = key.toString();
					appendLevel(ps, level + 1).append("<");
					xmlEscape(ps, keyStr).append(">");
					writeObject(map.get(key), ps, level + 1, false);
					ps.append("</");
					xmlEscape(ps, keyStr).append(">\n");
				}
				appendLevel(ps, level).append("</").append(className).append(">\n");
			}
		}
	}
}
