package com.grizzly.test.arp.client;

import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

public class ArpClient implements Runnable, MessageNotifier
{
	private static AtomicInteger invalidViewIdCount = new AtomicInteger(0);
	private static AtomicInteger badMessageCount = new AtomicInteger(0);
	private static AtomicInteger badHttpResponse = new AtomicInteger(0);

	private static ThreadPoolExecutor executor = new ThreadPoolExecutor(30, 30, 60, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>());

	public static void main(String[] args)
	{
		int viewId = 0;
		for (int i = 0; i < 500; i++)
		{
			viewId++;
			ArpClient requester = new ArpClient(viewId);
			executor.execute(requester);
		}

		try
		{
			executor.shutdown();
			executor.awaitTermination(2, TimeUnit.DAYS);
		}
		catch (InterruptedException e)
		{
			// Ok
		}

		String message = "invalidViewIdCount=" + invalidViewIdCount.get();
		message += ", badMessageCount=" + badMessageCount.get();
		message += ", badHttpResponse=" + badHttpResponse.get();
		System.out.println(message);

		System.exit(0);
	}

	private String viewId;
	private int messageCount;

	public ArpClient(int viewId)
	{
		this.viewId = "" + viewId;
	}

	@Override
	public void run()
	{
		try
		{
			refreshView();
		}
		catch (Throwable t)
		{
			System.out.println(t.getMessage());
			t.printStackTrace();
		}
	}

	public void refreshView()
	{
		int response = StreamRequest.GET("http://localhost:8080/refreshView?viewId=" + viewId, this, "text/xml");
		if (response != 200)
		{
			System.out.println(viewId + ": Error response=" + response);
			badHttpResponse.addAndGet(1);
		}
		else
		{
			if (messageCount != 30)
			{
				System.out.println(">>>> " + viewId + ": done. Got " + messageCount + " messages. >>> Bad message count <<<");
				badMessageCount.addAndGet(1);
			}
			else
			{
				System.out.println(viewId + ": done. Got " + messageCount + " messages.");
			}
		}
	}

	@Override
	public void onMessage(Object message)
	{
		String str = message.toString();
		String source = extractString(str, "<source>", "</source>");
		String msgId = extractString(str, "<name>", "</name>");
		if (msgId == null)
			msgId = extractString(str, "<id>", "</id>");

		if (source != null && source.equals("internal"))
		{
			// System.out.println(viewId+": Internal message #" + msgId);
		}
		else
		{
			String msgViewId = extractString(str, "<viewId>", "</viewId>");
			System.out.println(viewId + ": message=" + msgId + ", viewId=" + msgViewId);
			if (msgViewId != null && msgViewId.equals(viewId) == false)
			{
				System.out.println(viewId + ": message=" + msgId + ", viewId=" + msgViewId + " >>>> Invalid view id <<<<");
				invalidViewIdCount.addAndGet(1);
			}

			messageCount++;
		}
	}

	public String extractString(String text, String afterText, String untilText)
	{
		int afterTextIndex = text.indexOf(afterText);
		if (afterTextIndex == -1)
			return null;
		String tempStr = text.substring(afterTextIndex + afterText.length());
		int untilTextIndex = tempStr.indexOf(untilText);
		if (untilTextIndex == -1)
			return null;
		tempStr = tempStr.substring(0, untilTextIndex);
		return tempStr;
	}
}
