package com.grizzly.test.arp.server;

import java.util.concurrent.locks.ReentrantLock;

/**
 * Asynchronous message definition.
 * 
 * @author eVrignaud
 */
public class AsyncMessage
{
	private static int messageCount = 0;
	private static ReentrantLock countLock = new ReentrantLock();

	private String id;
	private String source;
	private String type;
	private Object data;

	/**
	 * Create an asynchronous message.
	 */
	public AsyncMessage()
	{
		computeMessageId();
	}

	/**
	 * Create an asynchronous message.
	 * 
	 * @param source The name of the sender of this message.
	 * @param type Message type.
	 * @param data Data object to be sent. This object can be a String or a Java Bean.
	 */
	public AsyncMessage(String source, String type, Object data)
	{
		this();
		this.source = source;
		this.type = type;
		this.data = data;
	}

	/**
	 * Recycle the message.
	 */
	public void recycle()
	{
		source = null;
		type = null;
		data = null;

		computeMessageId();
	}

	/** Return the message id */
	public String getId()
	{
		return id;
	}

	public void setId(String id)
	{
		// Does nothing
	}

	/** Return the name of the sender of this message. */
	public String getSource()
	{
		return source;
	}

	/** Set the name of the sender of this message. */
	public void setSource(String source)
	{
		this.source = source;
	}

	/** Return the message type. */
	public String getType()
	{
		return type;
	}

	/** Set the message type. */
	public void setType(String type)
	{
		this.type = type;
	}

	/** Return the data object to be sent. */
	public Object getData()
	{
		return data;
	}

	/** Set the data object to be sent. This object can be a String or a Java Bean. */
	public void setData(Object data)
	{
		this.data = data;
	}

	private void computeMessageId()
	{
		try
		{
			countLock.lock();

			messageCount++;
			id = "msg_" + messageCount;
		}
		finally
		{
			countLock.unlock();
		}
	}
}
