package com.grizzly.test.arp.server.util;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

public class ClassHelper
{
	/**
	 * Return the entire set of properties for which the specified bean
	 * provides a read method or public attributes.</p>
	 */
	public static Map<String, Object> describe(Object instance)
	{
		Map<String, Object> attribs = new HashMap<String, Object>();

		if (instance instanceof Map || instance instanceof Hashtable)
		{
			Map map = (Map) instance;
			Set keys = map.keySet();
			for (Object key : keys)
			{
				attribs.put(key.toString(), map.get(key));
			}
			return attribs;
		}

		try
		{
			Set<String> fieldNames = new HashSet<String>();

			// Add read/write properties (ie: public getter/setter).
			BeanInfo info = Introspector.getBeanInfo(instance.getClass());
			for (PropertyDescriptor property : info.getPropertyDescriptors())
			{
				if (property.getWriteMethod() != null && property.getReadMethod() != null)
				{
					try
					{
						String propName = property.getName();
						Object propValue = property.getReadMethod().invoke(instance);
						attribs.put(propName, propValue);
					}
					catch (Exception e)
					{
						// Never mind
					}
				}
			}

			// Add other public fields.
			Field[] fields = instance.getClass().getFields();
			for (Field field : fields)
			{
				String propertyName = field.getName();
				if (!fieldNames.contains(propertyName) && !Modifier.isStatic(field.getModifiers()) && !Modifier.isTransient(field.getModifiers()))
				{
					try
					{
						String propName = field.getName();
						Object propValue = field.get(instance);
						attribs.put(propName, propValue);
					}
					catch (Exception e)
					{
						// Never mind
					}
				}
			}
		}
		catch (IntrospectionException e)
		{
			System.out.println("Got exception: " + e.getMessage());
			e.printStackTrace();
		}

		return attribs;
	}
}
