package com.grizzly.test.arp.server.util;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

public abstract class TextWriter
{
	/** 
	* Append to the given PrintStream an escaped version of the
	* given text string where XML special characters have been escaped.
	* For a null string we append "<null>"
	*/
	protected PrintStream xmlEscape(PrintStream ps, String str)
	{
		if (str == null)
		{
			str = "<null>";
		}
		for (int i = 0; i < str.length(); i++)
		{
			char c = str.charAt(i);
			switch (c)
			{
			case '&':
				ps.append("&amp;");
				break;

			case '<':
				ps.append("&lt;");
				break;

			case '>':
				ps.append("&gt;");
				break;

			default:
				if (((c >= 0x20) && (c <= 0x7e)) || (c == /* line feed */10) || (c == /* carriage return */13))
				{
					ps.append(c);
				}
				else
				{
					ps.append("&#").print((int) c);
					ps.append(';');
				}
				break;
			}
		}
		return ps;
	}

	protected PrintStream appendLevel(PrintStream ps, int level)
	{
		for (int i = 0; i < level; i++)
		{
			ps.append("  ");
		}
		return ps;
	}

	protected boolean isNativeType(Object obj)
	{
		return obj.getClass().getPackage().getName().equals("java.lang");
	}

	protected List<String> getPrimitiveArrayAsList(Object data)
	{
		List<String> result = new ArrayList<String>();

		if (data instanceof boolean[])
		{
			boolean[] values = (boolean[]) data;
			for (boolean value : values)
			{
				result.add("" + value);
			}
		}
		else if (data instanceof byte[])
		{
			byte[] values = (byte[]) data;
			for (byte value : values)
			{
				result.add("" + value);
			}
		}
		else if (data instanceof char[])
		{
			char[] values = (char[]) data;
			for (char value : values)
			{
				result.add("" + value);
			}
		}
		else if (data instanceof double[])
		{
			double[] values = (double[]) data;
			for (double value : values)
			{
				result.add("" + value);
			}
		}
		else if (data instanceof float[])
		{
			float[] values = (float[]) data;
			for (float value : values)
			{
				result.add("" + value);
			}
		}
		else if (data instanceof int[])
		{
			int[] values = (int[]) data;
			for (int value : values)
			{
				result.add("" + value);
			}
		}
		else if (data instanceof long[])
		{
			long[] values = (long[]) data;
			for (long value : values)
			{
				result.add("" + value);
			}
		}
		else if (data instanceof short[])
		{
			short[] values = (short[]) data;
			for (short value : values)
			{
				result.add("" + value);
			}
		}
		else
		{
			return null;
		}
		return result;
	}
}
